<?php

namespace Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;
use Zeedhi\DevOps\Util\Utilities;

class IssueLabels
{
    use TQuery;

    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * IssueLabels constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function findMyIssueLabels($userID)
    {
        $query = <<<SQL
                SELECT * FROM (
                SELECT L.ID, L.RESOURCE_T, L.PROJECT, L.DESCRIPTION, P.NAME AS PROJECT_NAME, RP.RESOURCE_C, RP.TYPE, PT.LABEL
                FROM LABEL L
                LEFT JOIN PROJECT P
                        ON L.PROJECT = P.ID
                LEFT JOIN RESOURCE_PROJECT RP
                        ON RP.PROJECT = P.ID
                        AND RP.TYPE   = 'M'
                        AND RP.RESOURCE_C = :RESOURCE_T
                LEFT JOIN PROJECT_TYPE PT
                        ON PT.ID = P.PROJECT_TYPE
                WHERE L.RESOURCE_T = :RESOURCE_T
                    OR L.PROJECT IN (SELECT P1.ID
                                    FROM PROJECT P1, RESOURCE_PROJECT RP
                                    WHERE P1.STATUS <> 'D'
                                        AND P1.PROJECT_STATUS IN (SELECT TS.ID
                                                                    FROM TASK_STATUS TS
                                                                WHERE TS.IS_PROJECT = 'Y'
                                                                    AND TS.CONTROL NOT IN ('C', 'X'))

                                        AND P1.PROJECT_TYPE IN (SELECT PT.ID
                                                                FROM PROJECT_TYPE PT
                                                                WHERE PT.EXTERNALTYPE IN ('P','M'))
                                        AND RP.PROJECT = P1.ID
                                        AND RP.RESOURCE_C = :RESOURCE_T
                                        AND RP.TYPE = 'M'
                                    UNION
                                    SELECT P.ID
                                    FROM PROJECT P
                                    WHERE P.STATUS <> 'D'
                                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                                FROM PROJECT_TYPE PT
                                                                WHERE PT.EXTERNALTYPE IN ('P'))
                                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                                FROM TASK_STATUS TS
                                                                WHERE TS.IS_PROJECT = 'Y'
                                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                                        AND P.ID IN (SELECT PJ.ID
                                                    FROM PROJECT PJ
                                                    WHERE PJ.STATUS <> 'D'
                                                        AND PJ.STRUCTURE IN (SELECT S.ID
                                                                            FROM STRUCTURE S
                                                                            WHERE S.ID IN (SELECT ST.ID
                                                                                            FROM STRUCTURE ST, PERMISSION PE
                                                                                            WHERE ST.ID = PE.STRUCTURE
                                                                                                AND ST.STATUS <> 'D'
                                                                                                AND PE.TYPE = 'M'
                                                                                                AND PE.RESOURCE_T = :RESOURCE_T)
                                                                                AND S.STATUS <> 'D'
                                                                            UNION
                                                                            SELECT S.ID
                                                                            FROM STRUCTURE S
                                                                            WHERE S.STATUS <> 'D'
                                                                            START WITH S.PARENT IN (SELECT ST.ID
                                                                                                        FROM STRUCTURE ST, PERMISSION PE
                                                                                                    WHERE ST.ID = PE.STRUCTURE
                                                                                                        AND ST.STATUS <> 'D'
                                                                                                        AND PE.TYPE = 'M'
                                                                                                        AND PE.RESOURCE_T = :RESOURCE_T)
                                                                            CONNECT BY PRIOR S.ID = S.PARENT))
                                    UNION
                                    SELECT P.ID
                                    FROM PROJECT P
                                    WHERE P.STATUS <> 'D'
                                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                                FROM PROJECT_TYPE PT
                                                                WHERE PT.EXTERNALTYPE IN ('P'))
                                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                                FROM TASK_STATUS TS
                                                                WHERE TS.IS_PROJECT = 'Y'
                                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                                    START WITH P.PARENT IN (SELECT S.ID
                                                                FROM STRUCTURE S
                                                            WHERE S.ID IN (SELECT ST.ID
                                                                                FROM STRUCTURE ST, PERMISSION PE
                                                                            WHERE ST.ID = PE.STRUCTURE
                                                                                AND ST.STATUS <> 'D'
                                                                                AND PE.TYPE = 'M'
                                                                                AND PE.RESOURCE_T = :RESOURCE_T)
                                                                AND S.STATUS <> 'D'
                                                            UNION
                                                            SELECT S.ID
                                                                FROM STRUCTURE S
                                                            WHERE S.STATUS <> 'D'
                                                            START WITH S.PARENT IN (SELECT ST.ID
                                                                                        FROM STRUCTURE ST, PERMISSION PE
                                                                                        WHERE ST.ID = PE.STRUCTURE
                                                                                        AND ST.STATUS <> 'D'
                                                                                        AND PE.TYPE = 'M'
                                                                                        AND PE.RESOURCE_T = :RESOURCE_T)
                                                            CONNECT BY PRIOR S.ID = S.PARENT)
                                    CONNECT BY PRIOR P.ID = P.PARENT)
                            ORDER BY L.DESCRIPTION)
SQL;

        $params = array(
            'RESOURCE_T'        => $userID,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function findProjectProductAndMeeting($userId)
    {
		$query = <<<SQL
		SELECT * FROM (
SELECT P.NAME , P.ID, P.ID AS IDPROJECT, PT.LABEL AS TYPE_NAME
  FROM PROJECT P
  JOIN	PROJECT_TYPE PT ON (P.PROJECT_TYPE = PT.ID)
    WHERE P.ID IN (SELECT P1.ID
                       FROM PROJECT P1, RESOURCE_PROJECT RP
                      WHERE P1.STATUS <> 'D'
                        AND P1.PROJECT_STATUS IN (SELECT TS.ID
                                                    FROM TASK_STATUS TS
                                                   WHERE TS.IS_PROJECT = 'Y'
                                                     AND TS.CONTROL NOT IN ('C', 'X'))
                        AND P1.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P','M'))
                        AND RP.PROJECT = P1.ID
                        AND RP.RESOURCE_C = :RESOURCE_T
                        AND RP.TYPE = 'M'
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                        AND P.ID IN (SELECT PJ.ID
                                       FROM PROJECT PJ
                                      WHERE PJ.STATUS <> 'D'
                                        AND PJ.STRUCTURE IN (SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.ID IN (SELECT ST.ID
                                                                               FROM STRUCTURE ST, PERMISSION PE
                                                                              WHERE ST.ID = PE.STRUCTURE
                                                                                AND ST.STATUS <> 'D'
                                                                                AND PE.TYPE = 'M'
                                                                                AND PE.RESOURCE_T = :RESOURCE_T)
                                                                AND S.STATUS <> 'D'
                                                              UNION
                                                             SELECT S.ID
                                                               FROM STRUCTURE S
                                                              WHERE S.STATUS <> 'D'
                                                              START WITH S.PARENT IN (SELECT ST.ID
                                                                                        FROM STRUCTURE ST, PERMISSION PE
                                                                                       WHERE ST.ID = PE.STRUCTURE
                                                                                         AND ST.STATUS <> 'D'
                                                                                         AND PE.TYPE = 'M'
                                                                                         AND PE.RESOURCE_T = :RESOURCE_T)
                                                            CONNECT BY PRIOR S.ID = S.PARENT))
                      UNION
                     SELECT P.ID
                       FROM PROJECT P
                      WHERE P.STATUS <> 'D'
                        AND P.PROJECT_TYPE IN (SELECT PT.ID
                                                 FROM PROJECT_TYPE PT
                                                WHERE PT.EXTERNALTYPE IN ('P'))
                        AND P.PROJECT_STATUS IN (SELECT TS.ID
                                                   FROM TASK_STATUS TS
                                                  WHERE TS.IS_PROJECT = 'Y'
                                                    AND TS.CONTROL NOT IN ('C', 'X'))
                      START WITH P.PARENT IN (SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.ID IN (SELECT ST.ID
                                                                FROM STRUCTURE ST, PERMISSION PE
                                                               WHERE ST.ID = PE.STRUCTURE
                                                                 AND ST.STATUS <> 'D'
                                                                 AND PE.TYPE = 'M'
                                                                 AND PE.RESOURCE_T = :RESOURCE_T)
                                                 AND S.STATUS <> 'D'
                                               UNION
                                              SELECT S.ID
                                                FROM STRUCTURE S
                                               WHERE S.STATUS <> 'D'
                                               START WITH S.PARENT IN (SELECT ST.ID
                                                                         FROM STRUCTURE ST, PERMISSION PE
                                                                        WHERE ST.ID = PE.STRUCTURE
                                                                          AND ST.STATUS <> 'D'
                                                                          AND PE.TYPE = 'M'
                                                                          AND PE.RESOURCE_T = :RESOURCE_T)
                                             CONNECT BY PRIOR S.ID = S.PARENT)
                    CONNECT BY PRIOR P.ID = P.PARENT) )

SQL;
        $params = array(
            'RESOURCE_T' => $userId,
        );
        
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function getAllProjectIdModel($organization){

        $query = <<<SQL
                SELECT P.ID
                FROM PROJECT P
                WHERE P.ORGANIZATION = :organization
                AND P.STATUS = 'A'
                START WITH P.PARENT IN ( SELECT PRO.ID
                                           FROM PROJECT PRO
                                           JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
                                          WHERE PRT.EXTERNALTYPE IN ( 'Q' , 'F' )
                                            AND PRO.ORGANIZATION = :organization
                                            AND PRO.STATUS = 'A')
                CONNECT BY PRIOR P.ID = P.PARENT
SQL;
        $params = array(
            'organization' => $organization,
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}