<?php


namespace Repository;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities as ProjectmanagementEntities;
use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Model\Traits\TQuery;

class Project extends RepositoryBase
{

    use TQuery;

    const ENTITY_NAME = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project';
    /**
     * @var EntityManager
     */
    private $entityManager;

    public function __construct()
    {
        parent::__construct(self::ENTITY_NAME);
        $this->entityManager = InstanceProvider::getEntityManager();
    }

    /**
     * @param $resourceId
     * @return array
     */
    public function findProjectsByOperador($resourceId): array
    {
        $query = <<<SQL
            SELECT PRO.ID AS PROJECT_ID, PRO.NAME AS PROJECT_NAME
              FROM PROJECT PRO
              JOIN RESOURCE_PROJECT RSP ON RSP.PROJECT = PRO.ID AND RSP.TYPE = 'M'
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
              JOIN PRODUCT PRD ON PRD.PROJECT_ID = PRO.ID
             WHERE RESOURCE_C = :RESOURCE_ID
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CANCELED)
               AND PRT.EXTERNALTYPE IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
SQL;
        $params = array(
            'RESOURCE_PROJECT_TYPE_MANAGER' => ProjectmanagementEntities\ResourceProject::MANAGER,
            ':RESOURCE_ID' => $resourceId,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(
                ProjectmanagementEntities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT
            ),
            ':PROJECT_STATUS_ACTIVE' => ProjectmanagementEntities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CANCELED' => ProjectmanagementEntities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params, $paramTypes);
    }

    public function modelProjectForFilterTaskTracking($organizationId, $userId, $filterString = '') {
		$query = <<<SQL
            SELECT *
              FROM (SELECT PRO.ID,PRO.NAME, (CASE WHEN PTY.EXTERNALTYPE = :typeModelRestricted THEN 'Y' ELSE 'N' END ) AS IS_RESTRICTED,
		                   PTY.LABEL AS PROJECT_TYPE_NME
                      FROM PROJECT PRO
                      JOIN PROJECT_TYPE PTY ON(PTY.ID = PRO.PROJECT_TYPE AND PTY.EXTERNALTYPE in(:typeModelRestricted,:typeModelFree))
                      JOIN TASK_STATUS PST ON (PST.ID = PRO.PROJECT_STATUS AND PST.CONTROL NOT IN (:controlCanceled, :controlConcluded))
                     WHERE PRO.ORGANIZATION = :organization
                       AND PRO.STATUS = :isActive
                       AND EXISTS(SELECT REP.ID FROM RESOURCE_PROJECT REP WHERE REP.PROJECT = PRO.ID AND REP.RESOURCE_C = :userId)
                  ORDER BY PRO.NAME)
             WHERE 1 = 1
         $filterString
SQL;

		$params = array(
			'organization' => $organizationId,
			'typeModelRestricted' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED,
			'typeModelFree' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_FREE,
			'isActive' => Entities\Project::IS_ACTIVE,
			'userId' => $userId,
            'controlCanceled'    => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'controlConcluded'   => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
		);
		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
	}

}
