<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Util\Messages;
use Repository\TaskTracking as TaskTrackingRepository;

class TaskTracking
{

    private $entityManager;
    private $environment;
    private $taskTrackingRepository;

    /**
     * TaskTracking constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager, TaskTrackingRepository $taskTrackingRepository)
    {
        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = $entityManager;
        $this->taskTrackingRepository = $taskTrackingRepository;
    }

    public function findTaskTracking($dynamicFilter,$search, $searchIn, $find, $order, $page, $limit){
        $filters = [
            [
                "columnName"=>"STATUS",
                "value"=>"%A%",
                "operator"=>"LIKE",
                "isCustomFilter"=>true
            ],
            [
                "columnName"=>"PERIOD",
                "value"=>[
                    "21/07/2021",
                    "27/07/2023"
                ],
                "operator"=>"BETWEEN",
                "isCustomFilter"=>true
            ],
            [
                "columnName"=>"STATUS_RESOURCE",
                "value"=>"A",
                "operator"=>"=",
                "isCustomFilter"=>true
                ]
            ];
        $filters = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
        $project = false;
        $statusResource = false;
        $addFilter = array();
        $filterLabel = '';
        $filterIssue = '';
        foreach ($filters as $key => &$filter){
            if($key == 'LATE') {
                $late = $filter[0]['value'];
            }
            //if($key == 'TASK_NAME_FILTER') {
            //    $filter[0]['value'] = str_replace("'", "''", $filter[0]['value']);
            //}
            if($key == 'PROJECT_ID'){
                $project = false;
            }else if($key == 'STRUCTURE'){
                $filter[0]['value'] = DevOpsUtilities::getArrayFromArrayAssociative(InstanceProvider::getService()->getStructure()->getAllStructuresChildrens($filter[0]['value']));
            }else if($key == 'PERIOD'){
                $addFilter = array(
                    'PERIOD_END' =>
                    $filter);
                unSet($filters[$key]);
            }else if($key == 'RESPONSIBLE'){
                $structures = DevOpsUtilities::getArrayFromArrayAssociative(InstanceProvider::getService()->getStructure()->getAllStructuresChildrensByManager($filter['value']));
                $structureFilter = array(
                    'STRUCTURE' =>
                    array(
                    'value'=>$structures,
                    'operation'=>'IN',
                    ));
                array_push($filters,$structureFilter);
                unSet($filters[$key]);
            }else if($key == 'LABEL') {
                $values = $filter[0]['value'];
                $filterLabel = implode(",", $values);
                unSet($filters[$key]);
            }else if($key == 'ISSUE') {
                $values = $filter[0]['value'];
                $filterIssue = implode(",", $values);
                unSet($filters[$key]);
            }else if($key == 'STATUS_RESOURCE') {
                $statusResource = $filter[0]['value'];
                unSet($filters[$key]);
            }
        }
        $filters = array_merge($filters,$addFilter);
        //$filterString = DevOpsUtilities::createQueryFiltered($filters);
        $filterString = InstanceProvider::getRepository()->getLocalTask()->createDynamicFilter($filters, 'WHERE 1=1', false);
        if(!empty($late)){
            $lateTasks = InstanceProvider::getRepository()->getTask()->getTotalDelayedTasksByCurrentUser($this->environment->getCurrentUserId(),true);
            if(!empty($lateTasks)) {
                $tasks = implode(",", Utilities::getArrayFromAssociativeArray($lateTasks));
                $filterString .= $late == 'Y' ? " AND TASK_ID IN ($tasks)" : "AND TASK_ID NOT IN ($tasks)";
            }
        }
        if(!empty($search) && !empty($searchIn)){
            $conditionsSearchQuery = InstanceProvider::getRepository()->getLocalTask()->createSearch ($search, $searchIn);
        }
        $tasks = InstanceProvider::getRepository()->getLocalTask()->getAllTasksByProjectOnDemand($this->environment->getCurrentOrganizationId(),
        $filterString, $filterLabel, $filterIssue, $project,$statusResource, $conditionsSearchQuery, $page, $limit);
        foreach($tasks["data"] as &$task){
            if(empty($task['_hasChild'])){
                if(empty($task['TOTAL_APPROPRIATION']  )) {
                    $task['TOTAL_APPROPRIATION'] = 0;
                }
            }
        }
        return $tasks;
    }

    public function findAllStatusTaskTracking() {
		return InstanceProvider::getRepository()->getLocalTask()->findAllStatusTaskTracking($this->environment->getCurrentOrganizationId());
	}

    public function findAllIssuesFromFilterTaskTracking() {
		return InstanceProvider::getRepository()->getLocalTask()->getAllIssuesFromFilterTaskTracking($this->environment->getCurrentOrganizationId());
	}

    public function findLabelsByProjectTaskTracking(){
        
        return InstanceProvider::getRepository()->getLocalTask()->findLabelsByProjectTaskTracking($this->environment->getCurrentOrganizationId(),$this->environment->getCurrentUserId());
    }
}